import java.util.*;
import pc2.ex.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.mail.*;
import javax.mail.internet.*;

public class AutoMailJudgeClient extends JudgeClient {
  private boolean StatusUpdated;
  private String TargetJudgeID;
  private java.util.Random randomGenerator;
  private HashMap sourceIdDict, submitIdDict;

  public AutoMailJudgeClient()
  {
    super();
    randomGenerator = new java.util.Random();
    sourceIdDict = new HashMap();
    submitIdDict = new HashMap();
  }

  public void submit(String sourceId,
                     String problemNumber,
                     String uvaId,
                     String language,
                     String comment,
                     String fileName,
                     String fileContent) {
    HTTPValladolid.Post(problemNumber,
                        uvaId,
                        language,
                        comment,
                        fileName,
                        "/* " + sourceId + " */\n" + fileContent);
  }

  public void SendMail(String fileContent,
                       String JudgeId,
                       String ProblemNumber,
                       String Language,
                       String sourceId) throws Exception
  {
    java.util.Properties props = System.getProperties();
    props.put("mail.smtp.host", "localhost");
    javax.mail.Session session =
      javax.mail.Session.getDefaultInstance(props, null);
    MimeMessage mimeMessage = new MimeMessage(session);
    mimeMessage.setFrom(new InternetAddress("mikurube@e.cc.titech.ac.jp",
                                            "Dai MIKURUBE", "iso-2022-jp"));
    mimeMessage.setRecipients(Message.RecipientType.TO, "judge@uva.es");

    mimeMessage.setText("/* @JUDGE_ID: " + JudgeId +
                        " " + ProblemNumber +
                        " " + Language +
                        " */\r\n/* " + sourceId +
                        " */\r\n" + fileContent, "iso-2022-jp");
    mimeMessage.setSentDate(new java.util.Date());
    javax.mail.Transport.send(mimeMessage);
  }

  public boolean hasAddress(Address[] Addresses, String Target)
  {
    for (int i = 0; i < Addresses.length; i++) {
      try {
        if (MimeUtility.decodeText(Addresses[i].toString()).indexOf(Target) >= 0) {
          return true;
        }
      } catch (java.io.UnsupportedEncodingException e) { continue; }
    }
    return false;
  }

  public String WaitForReceivedMail(String sourceId) throws Exception
  {
    javax.mail.Session session =
      javax.mail.Session.getDefaultInstance(System.getProperties(), null);
    Store store = session.getStore("pop3");
    store.connect("yad-mail.local", -1, "dms", "mlvn3925");

    Folder folder = store.getFolder("INBOX");
    folder.open(Folder.READ_WRITE);

    Message[] messages = folder.getMessages();
    for (int i = 0; i < messages.length; i++) {
      if (!hasAddress(messages[i].getFrom(), "lawyer@acm.uva.es")) { continue; }

      if (messages[i].getContent().toString().indexOf(sourceId) >= 0) {
        System.out.println("    ReceivedMail: " + messages[i].getSubject());
        messages[i].setFlag(Flags.Flag.DELETED, true);
        folder.close(true);
        store.close();
        return messages[i].getSubject().substring(0, 5);
      }
    }
    folder.close(true);
    store.close();
    return null;
  }

  public int WaitForResultMail(String submitId) throws Exception
  {
    String ResultMessages[] =
    {"Accepted",
     "Compile Error",
     "Runtime",
     "Time Limit",
     "Wrong Answer",
     "Output Limit",
     "Presentation"};

    javax.mail.Session session =
      javax.mail.Session.getDefaultInstance(System.getProperties(), null);
    Store store = session.getStore("pop3");
    store.connect("yad-mail.local", -1, "dms", "mlvn3925");

    Folder folder = store.getFolder("INBOX");
    folder.open(Folder.READ_WRITE);

    Message[] messages = folder.getMessages();

    int result = -1;
  AllMessages:
    for (int i = 0; i < messages.length; i++) {
      if (!hasAddress(messages[i].getFrom(), "lawyer@acm.uva.es")) { continue; }
      if (messages[i].getSubject().indexOf(submitId) < 0) { continue; }

      System.out.println("      ResultMail: " + messages[i].getSubject());
      messages[i].setFlag(Flags.Flag.DELETED, true);
      for (int j = 0; j < ResultMessages.length; j++) {
        if (messages[i].getSubject().indexOf(ResultMessages[j]) >= 0) {
          result = j;
          break AllMessages;
        }
      }
      result = ResultMessages.length;
      break;
    }
    folder.close(true);
    store.close();
    return result;
  }

  public String getFileContent(RunInfoAndFiles run)
  {
int i = 0;
    if (run.getNumUserFiles() <= 0) { return null; }
    try {
      if (!run.writeUserFile(0, run.getUserFileName(0))) { return null; }
    } catch (Exception e) { return null; }

    String fileContent = "";

    java.io.FileReader reader;
    try {
      reader = new java.io.FileReader(run.getUserFileName(0));
    } catch (java.io.FileNotFoundException e) { return null; }
    while (true) {
      int character;
      try {
        character = reader.read();
      } catch (java.io.IOException e) { return fileContent; }
      if (0 > character) { break; }
      fileContent += (char)character;
    }
    try {
      reader.close();
    } catch (java.io.IOException e) { return fileContent; }

    return fileContent;
  }

  public void JudgeAll() throws Exception
  {
    RunInfoAndFiles runs[] = getAllUnjudgedRuns();
    String sourceId = "@SOURCE_ID: " + java.lang.Long.toString(randomGenerator.nextLong(), 36);

    for (int i = 0; i < runs.length; i++) {
      RunInfoAndFiles target_run;
      System.out.println("Judgeing...: UserSite=" + runs[i].getUserSiteId() +
                         " User=" + runs[i].getUserId() +
                         " Run=" + runs[i].getRunId());

      try {
        target_run = checkOutRun(new java.lang.Long(runs[i].getUserSiteId()),
                                 new java.lang.Long(runs[i].getRunId()));
      } catch (Exception e) { continue; }

      String fileContent = getFileContent(target_run);
      if (fileContent == null) { continue; }
      submit(sourceId,
             target_run.getProblemTitle(),
             "28595JR",
             target_run.getLanguageTitle(),
             "",
             target_run.getUserFileName(0),
             fileContent);
      /*
      SendMail(fileContent, TargetJudgeID,
               target_run.getProblemTitle(),
               target_run.getLanguageTitle(),
               sourceId);
      */

      String submitId;
      while (null == (submitId = WaitForReceivedMail(sourceId))) {
        java.lang.Thread.sleep(3000);
      } System.out.println("    Program Received: " + submitId);

      int result;
      while (0 > (result = WaitForResultMail(submitId))) {
        java.lang.Thread.sleep(3000);
      } System.out.println("    Result: " + result);

      if (result == 0) { submitJudgement(target_run, 0,      true);  }
      else             { submitJudgement(target_run, result, false); }
    }
  }

  public static void main(String[] args) throws Exception {
    AutoMailJudgeClient Client = new AutoMailJudgeClient();
    String JudgeLoginID = "judge1";
    if (args.length > 0) { JudgeLoginID = args[0]; }
    Client.TargetJudgeID = "28595JR";
    if (args.length > 1) { Client.TargetJudgeID = args[1]; }

    System.out.println("" + Client.loginToPC2("localhost", "50002", 0,
                                              JudgeLoginID,
                                              JudgeLoginID));

    /* --- GUI Settings --- */
    Client.setBounds(10, 10, 300, 200);
    Container contentPane = Client.getContentPane();
    contentPane.setLayout(new GridLayout(2, 1));
    JButton ExitButton = new JButton("Exit");
    ExitButton.addActionListener(new ExitActionListener(Client));
    contentPane.add(ExitButton);
    Client.pack();
    Client.setVisible(true);

    for (int i = 1; i <= Client.getNumLanguages(); i++) {
      System.out.println(Client.getLanguageTitle(new java.lang.Long(i)));
    }

    Client.StatusUpdated = true;
    while (true) {
      if (Client.StatusUpdated) {
        Client.StatusUpdated = false;
        Client.JudgeAll();
      }
      java.lang.Thread.sleep(3000);
    }

    //    Client.logoffPC2();
  }





  public void handleNewRun(RunInfoAndFiles run)
  {
    //    System.out.println("        handleNewRun()");
  }

  public void handleSettingsUpdated()
  {
    //    System.out.println("        handleSettingsUpdated()");
  }

  public void handleCheckedOutRun(RunInfoAndFiles run)
  {
    //    System.out.println("        handleCheckedOutRun()");
  }

  public void handleClarAnswered(ClarInfoAndFiles clar)
  {
    //    System.out.println("        handleClarAnswerd()");
  }

  public void handleNewClar(ClarInfoAndFiles clar)
  {
    //    System.out.println("        handleNewClar()");
  }

  public void handleRunJudged(RunInfoAndFiles run, boolean isSolved, String judgement)
  {
    //    System.out.println("        handleRunJudged()");
  }

  public void handleRunStatusUpdated(RunInfoAndFiles run)
  {
    //    System.out.println("        handleRunStatusUpdated()");
  }

  public void handleRunUpdated(RunInfoAndFiles run)
  {
    //    System.out.println("        handleRunUpdated()");
  }

  public void handleUNCheckedOutRun(RunInfoAndFiles run)
  {
    StatusUpdated = true;
    //    System.out.println("        handleUNCheckedOutRun()");
  }

  public void handleClarSelected(ClarInfoAndFiles clar)
  {
    //    System.out.println("        handleClarSelected()");
  }

  public void handleClientRefresh()
  {
    //    System.out.println("        handleClientRefresh()");
  }
}

class ExitActionListener implements java.awt.event.ActionListener {
  pc2.ex.JudgeClient client;
  public ExitActionListener(pc2.ex.JudgeClient client) {
    this.client = client;
  }
  public void actionPerformed(ActionEvent event)
  {
    client.logoffPC2();
    System.exit(0);
  }
}
