import java.util.*;
import pc2.ex.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.mail.*;
import javax.mail.internet.*;

public class UVAJudge extends JudgeClient {
  private java.util.Random randomGenerator;
  private UVAJudgeFrame uvaJudgeFrame;
  private UVARunTableModel uvaRunTableModel;

  private String mailServer, mailUser, mailPassword;

  public UVAJudge(String pc2User, String pc2Password,
                  String mailServer, String mailUser, String mailPassword)
                  throws LoginFailureException
  {
    super();
    try {
      if (!loginToPC2("localhost", "50002", 0, pc2User, pc2Password)) {
        throw new LoginFailureException();
      }
    } catch (Exception e) { throw new LoginFailureException(); }

    this.mailServer = mailServer;
    this.mailUser = mailUser;
    this.mailPassword = mailPassword;

    randomGenerator = new java.util.Random();
    uvaRunTableModel = new UVARunTableModel();
    uvaJudgeFrame = new UVAJudgeFrame(this,
                                      uvaRunTableModel,
                                      "PC^2 UVAJudge Main");
    uvaJudgeFrame.pack();
    uvaJudgeFrame.setVisible(true);
  }

  public void submit(String sourceId,
                     String problemNumber,
                     String uvaId,
                     String language,
                     String comment,
                     String fileName,
                     String fileContent) {
    HTTPValladolid.Post(problemNumber,
                        uvaId,
                        language,
                        comment,
                        fileName,
                        "/* " + sourceId + " */\n" + fileContent);
  }

  public boolean hasAddress(Address[] addresses, String target)
  {
    for (int i = 0; i < addresses.length; i++) {
      try {
        if (MimeUtility.decodeText(addresses[i].toString()).indexOf(target)
            >= 0) {
          return true;
        }
      } catch (java.io.UnsupportedEncodingException e) { continue; }
    }
    return false;
  }

  private void CheckForMail() {
    String resultMessages[] =
    {"Accepted",
     "Compile Error",
     "Runtime",
     "Time Limit",
     "Wrong Answer",
     "Output Limit",
     "Presentation",
     ""};

    javax.mail.Session session =
      javax.mail.Session.getDefaultInstance(System.getProperties(), null);
    Store store;
    try {
      store = session.getStore("pop3");
    } catch (javax.mail.NoSuchProviderException e) { return; }
    try {
      store.connect(mailServer, -1, mailUser, mailPassword);
    } catch (javax.mail.MessagingException e) { return; }

    Folder folder;
    try {
      folder = store.getFolder("INBOX");
      folder.open(Folder.READ_WRITE);
    } catch (javax.mail.MessagingException e) { return; }
    
    Message[] messages;
    try {
      messages = folder.getMessages();
    } catch (javax.mail.MessagingException e) { return; }
    for (int i = 0; i < messages.length; i++) {
      String mailSubject, mailContent;
      Address[] mailFromAddresses;
      try {
        mailSubject = messages[i].getSubject();
        mailContent = messages[i].getContent().toString();
        mailFromAddresses = messages[i].getFrom();
      }
      catch (javax.mail.MessagingException e) { continue; }
      catch (java.io.IOException e) { continue; }

      if (!hasAddress(mailFromAddresses, "lawyer@acm.uva.es")) { continue; }

      for (int j = 0; j < uvaRunTableModel.getNumRunInfo(); j++) {
        UVARunInfoAndFiles uvaRun = uvaRunTableModel.getRunInfo(j);

        if (uvaRun.getStatus() == UVARunInfoAndFiles.STATUS_SUBMITTED &&
            mailSubject.indexOf("Program Received") >= 0 &&
            mailContent.indexOf(uvaRun.getSourceId()) >= 0) {
          try {
            messages[i].setFlag(Flags.Flag.DELETED, true);
          } catch (javax.mail.MessagingException e) { }
          uvaRunTableModel.setProgramReceived(j, mailSubject.substring(0, 5));
          continue;
        }

        if (uvaRun.getStatus() == UVARunInfoAndFiles.STATUS_PROGRAM_RECEIVED &&
            mailSubject.startsWith(uvaRun.getSubmitId())) {
          if (mailSubject.indexOf("Program Received") >= 0) { continue; }

          for (int k = 0; k < resultMessages.length; k++) {
            if (mailSubject.indexOf(resultMessages[k]) >= 0) {
              try {
                messages[i].setFlag(Flags.Flag.DELETED, true);
              } catch (javax.mail.MessagingException e) { }
              uvaRunTableModel.setJudged(j, k);
              RunInfoAndFiles run;
              try {
                run = checkOutRun(new Long(uvaRunTableModel.getRunInfo(j).
                                           getUserSiteId()),
                                  new Long(uvaRunTableModel.getRunInfo(j).
                                           getRunId()));
                if (k == 0) { submitJudgement(run, k, true); }
                else        { submitJudgement(run, k, false); }
              } catch (Exception e) { break; }
              break;
            }
          }
        }
            
      }
    }
    try { folder.close(true); } catch (javax.mail.MessagingException e) {}
    try { store.close(); }  catch (javax.mail.MessagingException e) {}
  }

  public String getFileContent(RunInfoAndFiles run)
  {
    if (run.getNumUserFiles() <= 0) { return null; }
    try {
      if (!run.writeUserFile(0, run.getUserFileName(0))) { return null; }
    } catch (Exception e) { return null; }

    String fileContent = "";
    java.io.FileReader reader;
    try {
      reader = new java.io.FileReader(run.getUserFileName(0));
    } catch (java.io.FileNotFoundException e) { return null; }
    while (true) {
      int character;
      try {
        character = reader.read();
      } catch (java.io.IOException e) { return fileContent; }
      if (0 > character) { break; }
      fileContent += (char)character;
    }
    try {
      reader.close();
    } catch (java.io.IOException e) { return fileContent; }

    return fileContent;
  }

  public void handleUNCheckedOutRun(RunInfoAndFiles run)
  {
    for (int i = 0; i < uvaRunTableModel.getNumRunInfo(); i++) {
      if (uvaRunTableModel.getRunInfo(i).getUserSiteId() ==
          run.getUserSiteId() &&
          uvaRunTableModel.getRunInfo(i).getRunId() ==
          run.getRunId()) {
        return;
      }
    }

    String sourceId =
      "@SOURCE_ID: " +
      java.lang.Long.toString(randomGenerator.nextLong(), 36);
    try {
      checkOutRun(new java.lang.Long(run.getUserSiteId()),
                  new java.lang.Long(run.getRunId()));
    } catch (Exception e) { return; }

    String fileContent = getFileContent(run);
    if (fileContent == null) { return; }
    submit(sourceId,
           run.getProblemTitle(),
           "28595JR",
           run.getLanguageTitle(),
           "",
           run.getUserFileName(0),
           fileContent);
    uvaRunTableModel.addRunInfo(run.getUserId(),
                                run.getUserSiteId(),
                                run.getRunId(),
                                sourceId);

    try {
      unCheckOutRun(run);
    } catch (Exception e) { return; }
  }

  public static void main(String[] args) {
    UVAJudge client;
    while (true) {
      UVAJudgeLogin.LoginInfo loginInfo = UVAJudgeLogin.loginDialog();
      try {
        client = new UVAJudge(loginInfo.pc2User, loginInfo.pc2Password,
                              loginInfo.mailServer,
                              loginInfo.mailUser, loginInfo.mailPassword);
      } catch (LoginFailureException e) { continue; }
      break;
    }

    while (true) {
      client.CheckForMail();
      try {
        java.lang.Thread.sleep(3000);
      } catch (InterruptedException e) { continue; }
    }
  }

  public void handleNewRun(RunInfoAndFiles run) {}
  public void handleSettingsUpdated() {}
  public void handleCheckedOutRun(RunInfoAndFiles run) {}
  public void handleClarAnswered(ClarInfoAndFiles clar) {}
  public void handleNewClar(ClarInfoAndFiles clar) {}
  public void handleRunJudged(RunInfoAndFiles run,
                              boolean isSolved, String judgement) {}
  public void handleRunStatusUpdated(RunInfoAndFiles run) {}
  public void handleRunUpdated(RunInfoAndFiles run) {}
  public void handleClarSelected(ClarInfoAndFiles clar) {}
  public void handleClientRefresh() {}

  /*
    class ExitActionListener implements java.awt.event.ActionListener {
    UVAJudge client;
    public ExitActionListener(UVAJudge client) {
      this.client = client;
    }
    public void actionPerformed(ActionEvent event) {
      client.logoffPC2();
      System.exit(0);
    }
  }
  */

  class LoginFailureException extends Exception {
    public LoginFailureException() { super(); }
    public LoginFailureException(String message) { super(message); }
  }
}
